from ka_uts_com.com import Com
from typing import Any
TyDic = dict[Any, Any]


class Log:
    """Logging Class
    """
    class Eq:

        @classmethod
        def error(cls, key: Any, value: Any) -> None:
            Log.error(f"{key} = {value}", stacklevel=3)

        @classmethod
        def warning(cls, key: Any, value: Any) -> None:
            Log.warning(f"{key} = {value}", stacklevel=3)

        @classmethod
        def info(cls, key: Any, value: Any) -> None:
            Log.info(f"{key} = {value}", stacklevel=3)

        @classmethod
        def log(cls, key: Any, value: Any) -> None:
            Log.log(f"{key} = {value}", stacklevel=3)

        @classmethod
        def debug(cls, key: Any, value: Any) -> None:
            Log.debug(f"{key} = {value}", stacklevel=3)

    class Dic:

        @classmethod
        def debug(cls, dic: TyDic) -> None:
            for key, value in dic.items():
                Log.debug(f"{key} = {value}", stacklevel=3)

        @classmethod
        def error(cls, dic: TyDic) -> None:
            for key, value in dic.items():
                Log.error(f"{key} = {value}", stacklevel=3)

        @classmethod
        def info(cls, dic: TyDic) -> None:
            for key, value in dic.items():
                Log.info(f"{key} = {value}", stacklevel=3)

        @classmethod
        def warning(cls, dic: TyDic) -> None:
            for key, value in dic.items():
                Log.warning(f"{key} = {value}", stacklevel=3)

    @staticmethod
    def error(*args, **kwargs) -> None:
        if kwargs is None:
            kwargs = {}
        kwargs['stacklevel'] = kwargs.get('stacklevel', 2)
        Com.Log.error(*args, **kwargs)

    @staticmethod
    def warning(*args, **kwargs) -> None:
        if kwargs is None:
            kwargs = {}
        kwargs['stacklevel'] = kwargs.get('stacklevel', 2)
        Com.Log.warning(*args, **kwargs)

    @staticmethod
    def info(*args, **kwargs) -> None:
        if kwargs is None:
            kwargs = {}
        kwargs['stacklevel'] = kwargs.get('stacklevel', 2)
        Com.Log.info(*args, **kwargs)

    @staticmethod
    def log(*args, **kwargs) -> None:
        if kwargs is None:
            kwargs = {}
        kwargs['stacklevel'] = kwargs.get('stacklevel', 2)
        Com.Log.log(*args, **kwargs)

    @staticmethod
    def debug(*args, **kwargs) -> None:
        if kwargs is None:
            kwargs = {}
        kwargs['stacklevel'] = kwargs.get('stacklevel', 2)
        Com.Log.debug(*args, **kwargs)



# coding=utf-8
from collections.abc import Callable
from typing import Any

TyArr = list[Any]
TyCallable = Callable[..., Any]
TyDic = dict[Any, Any]
TyArrDic = TyArr | TyDic
TyDoC = dict[str, TyCallable]
TyMsg = str

TnStr = None | str
TnDoC = None | TyDoC
TnCallable = None | TyCallable


class Fnc:
    """
    Functions
    """
    @staticmethod
    def identity(obj: Any) -> Any:
        return obj

    @staticmethod
    def sh(doc: TnDoC, key: TnStr) -> TyCallable:
        if not doc:
            msg = f"function table: {doc} is not defined"
            raise Exception(msg)
        if not key:
            msg = f"key: {key} is not defined"
            raise Exception(msg)
        fnc: TnCallable = doc.get(key)
        if not fnc:
            msg = f"key: {key} is not defined in function table: {doc}"
            raise Exception(msg)
        return fnc

    @classmethod
    def ex(cls, doc: TnDoC, key: TnStr, args_kwargs: TyArrDic) -> Any:
        fnc: TyCallable = cls.sh(doc, key)
        return fnc(args_kwargs)



# coding=utf-8
from typing import Any

import os
import time
import calendar
import importlib.resources as resources
import logging
import logging.config
from logging import Logger
from datetime import datetime

from ka_uts_com.utils.aoeqstmt import AoEqStmt
from ka_uts_com.utils.pacmod import PacMod
from ka_uts_com.base.app_ import App_
from ka_uts_com.base.cfg_ import Cfg_
from ka_uts_com.base.exit_ import Exit_
from ka_uts_com.base.log_ import Log_

TyAny = Any
TyDateTime = datetime
TyTimeStamp = int
TyArr = list[Any]
TyBool = bool
TyDic = dict[Any, Any]
TyLogger = Logger

TnAny = None | Any
TnArr = None | TyArr
TnDic = None | TyDic
TnTimeStamp = None | TyTimeStamp
TnDateTime = None | TyDateTime
TnStr = None | str


class Com:
    """Communication Class
    """
    sw_init: bool = False
    dir_dat: TnStr = None
    tenant: TnStr = None
    log_type: TnStr = None
    cmd: TnStr = None
    d_com_pacmod: TyDic = {}
    d_app_pacmod: TyDic = {}
    path_bin = None
    path_log_cfg = None

    pid = None

    ts: TnTimeStamp
    ts_start: TnDateTime = None
    ts_end: TnDateTime = None
    ts_etime: TnDateTime = None
    d_timer: TyDic = {}

    cfg: TnDic = None
    Log = logging.getLogger('dummy_logger')
    App: Any = None
    Exit: Any = None

    @classmethod
    def init(cls, app_cls, kwargs: TyDic):
        """ set log and application (module) configuration
        """
        if cls.sw_init:
            return
        cls.sw_init = True
        cls.dir_dat = kwargs.get('dir_dat', '/data')
        cls.tenant = kwargs.get('tenant')
        cls.log_type = kwargs.get('log_type', 'std')
        cls.cmd = kwargs.get('cmd')
        cls.d_com_pacmod = PacMod.sh_d_pacmod(cls)
        cls.d_app_pacmod = PacMod.sh_d_pacmod(app_cls)

        cls.ts = calendar.timegm(time.gmtime())
        cls.pid = os.getpid()

        # print(f"Com.init cls.tenanat = {cls.tenant}")
        # print(f"Com.init cls.cmd = {cls.cmd}")
        # print(f"Com.init kwargs = {kwargs}")

        cls.set_path_bin()
        cls.set_path_log_cfg()

        cls.Log = Log_.sh(cls, **kwargs)
        cls.cfg = Cfg_.sh(cls, **kwargs)
        cls.App = App_.sh(cls, **kwargs)
        cls.Exit = Exit_.sh(**kwargs)

    @classmethod
    def sh_kwargs(cls, app_cls, d_parms, *args) -> TyDic:
        _kwargs: TyDic = AoEqStmt.sh_d_eq(*args, d_parms=d_parms)
        cls.init(app_cls, _kwargs)
        _kwargs['com'] = cls
        return _kwargs

    @classmethod
    def set_path_bin(cls):
        """ show directory
        """
        package = cls.d_app_pacmod['package']
        path = resources.files(package).joinpath("bin")
        if path.is_file():
            cls.path_bin = path
        # raise ModuleNotFoundError

    @classmethod
    def set_path_log_cfg(cls):
        """ show directory
        """
        package = cls.d_app_pacmod['package']
        path = resources.files(package).joinpath(f"data/log.{cls.log_type}.yml")
        if path.is_file():
            cls.path_log_cfg = path
            return
        package = cls.d_com_pacmod['package']
        path = resources.files(package).joinpath(f"data/log.{cls.log_type}.yml")
        if path.is_file():
            cls.path_log_cfg = path
        # raise ModuleNotFoundError



# coding=utf-8
from collections.abc import Callable
from typing import Any

import os
import logging
import logging.config
from logging import Logger
from datetime import datetime
import psutil

# from ka_uts_com.com import Com
from ka_uts_com.ioc.jinja2_ import Jinja2_

TyAny = Any
TyCallable = Callable[..., Any]
TyDateTime = datetime
TyTimeStamp = int
TyArr = list[Any]
TyBool = bool
TyDic = dict[Any, Any]
TyDir = str
TyPath = str
TyLogger = Logger

TnAny = None | Any
TnArr = None | TyArr
TnBool = None | bool
TnDic = None | TyDic
TnTimeStamp = None | TyTimeStamp
TnDateTime = None | TyDateTime


class Log_:

    sw_init: bool = False
    log: TyLogger = logging.getLogger('dummy_logger')
    username: str = psutil.Process().username()

    @classmethod
    def sh_dir_run(cls, com) -> TyDir:
        """Show run_dir
        """
        # print(f"sh_dir_run com.dir_dat = {com.dir_dat}")
        # print(f"sh_dir_run com.cmd = {com.cmd}")
        # print(f"sh_dir_run com.tenant = {com.tenant}")
        # print(f"sh_dir_run com = {com}")
        dir_dat: str = com.dir_dat
        tenant: str = com.tenant
        cmd: str = com.cmd
        package: str = com.d_app_pacmod['package']
        # module: str = com.d_app_pacmod['module']
        if tenant is not None:
            path = f"{dir_dat}/{tenant}/RUN/{package}"
        else:
            path = f"{dir_dat}/RUN/{package}"
        if com.log_type == "usr":
            path = f"{path}/{cls.username}"
        if cmd is not None:
            path = f"{path}/{cmd}"
        # print(f"sh_dir_run path = {path}")
        # print("==================================")
        return path

    @classmethod
    def sh_d_log_cfg(cls, com, **kwargs) -> TyDic:
        """Read log file path with jinja2
        """
        dir_run = cls.sh_dir_run(com)
        if kwargs.get('sw_single_dir_run', True):
            # print("---sw_single_dir_run = True --------------")
            dir_run_errs = f"{dir_run}/logs"
            dir_run_wrns = f"{dir_run}/logs"
            dir_run_infs = f"{dir_run}/logs"
            dir_run_logs = f"{dir_run}/logs"
            dir_run_debs = f"{dir_run}/logs"
            if kwargs.get('sw_mk_dir_run', True):
                os.makedirs(dir_run_logs, exist_ok=True)
        else:
            # print("---sw_single_dir_run = False -------------")
            dir_run_errs = f"{dir_run}/errs"
            dir_run_wrns = f"{dir_run}/wrns"
            dir_run_infs = f"{dir_run}/infs"
            dir_run_logs = f"{dir_run}/logs"
            dir_run_debs = f"{dir_run}/debs"
            if kwargs.get('sw_mk_dir_run', True):
                os.makedirs(dir_run_errs, exist_ok=True)
                os.makedirs(dir_run_wrns, exist_ok=True)
                os.makedirs(dir_run_infs, exist_ok=True)
                os.makedirs(dir_run_logs, exist_ok=True)
                os.makedirs(dir_run_debs, exist_ok=True)
        # path_log_cfg: TyPath = PacMod.sh_path_log_cfg(com)
        module = com.d_app_pacmod['module']
        # print(f"sh_d_log_cfg cls.path_log_cfg = {cls.path_log_cfg}")
        d_log_cfg: TyDic = Jinja2_.read(
                com.path_log_cfg, com.Log,
                dir_run_errs=dir_run_errs,
                dir_run_wrns=dir_run_wrns,
                dir_run_infs=dir_run_infs,
                dir_run_logs=dir_run_logs,
                dir_run_debs=dir_run_debs,
                module=module,
                pid=com.pid,
                ts=com.ts)
        # print(f"sh_d_log_cfg d_log_cfg = {d_log_cfg}")
        sw_debug: TyBool = kwargs.get('sw_debug', False)
        if sw_debug:
            level = logging.DEBUG
        else:
            level = logging.INFO
        logger_name = com.log_type
        d_log_cfg['handlers'][f"{logger_name}_debug_console"]['level'] = level
        d_log_cfg['handlers'][f"{logger_name}_debug_file"]['level'] = level

        return d_log_cfg

    @classmethod
    def init(cls, com, **kwargs) -> None:
        """Set static variable log level in log configuration handlers
        """
        cls.sw_init = True
        d_log_cfg = cls.sh_d_log_cfg(com, **kwargs)
        logging.config.dictConfig(d_log_cfg)
        cls.log = logging.getLogger(com.log_type)

    @classmethod
    def sh(cls, com, **kwargs) -> TyLogger:
        if cls.sw_init:
            return cls.log
        cls.init(com, **kwargs)
        return cls.log



# coding=utf-8
from collections.abc import Callable
from typing import Any

# from ka_uts_com.utils.pacmod import PacMod
from ka_uts_com.utils.str import Str
from ka_uts_com.utils.date import Date


TyArr = list[Any]
TyCall = Callable[..., Any]
TyDic = dict[Any, Any]
TyStr = str

TnArr = None | TyArr
TnDic = None | TyDic
TnStr = None | TyStr


class DoEq:
    """ Manage Commandline Arguments
    """
    @classmethod
    def sh_value(cls, key: str, value: Any, d_valid_parms: TnDic) -> Any:

        # print(f"key = {key}, type(key) = {type(key)}")
        # print(f"value = {value}, type(value) = {type(value)}")
        if not d_valid_parms:
            return value
        _type: TnStr = d_valid_parms.get(key)
        # print(f"_type = {_type}")
        if not _type:
            return value
        if isinstance(_type, str):
            match _type:
                case 'int':
                    value = int(value)
                case 'bool':
                    value = Str.sh_boolean(value)
                case 'dict':
                    value = Str.sh_dic(value)
                case 'list':
                    value = Str.sh_arr(value)
                case '%Y-%m-%d':
                    value = Date.sh(value, _type)
                case '_':
                    match _type[0]:
                        case '[', '{':
                            _obj = Str.sh_dic(_type)
                            if value not in _obj:
                                msg = (f"parameter={key} value={value} is invalid; "
                                       f"valid values are={_obj}")
                                raise Exception(msg)

        # print(f"value = {value}, type(value) = {type(value)}")
        return value

    # @staticmethod
    # def _set_d_pacmod(d_eq: TyDic, root_cls) -> None:
    #   """ set current pacmod dictionary
    #   """
    #   tenant = d_eq.get('tenant')
    #   d_eq['d_pacmod'] = PacMod.sh_d_pacmod(root_cls, tenant)

    @staticmethod
    def _set_sh_prof(d_eq: TyDic, sh_prof: TyCall | Any) -> None:
        """ set current pacmod dictionary
        """
        if callable(sh_prof):
            d_eq['sh_prof'] = sh_prof()
        else:
            d_eq['sh_prof'] = sh_prof

    @classmethod
    def verify(cls, d_eq: TyDic, d_parms: TnDic) -> TyDic:
        if d_parms is None:
            return d_eq
        if 'cmd' in d_eq:
            _d_valid_parms = d_parms
            _cmd = d_eq['cmd']
            _valid_commands = list(d_parms.keys())
            if _cmd not in _valid_commands:
                msg = (f"Wrong command: {_cmd}; "
                       f"valid commands are: {_valid_commands}")
                raise Exception(msg)
            _d_valid_parms = d_parms[_cmd]
        else:
            _d_valid_parms = d_parms
        if _d_valid_parms is None:
            return d_eq

        d_eq_new = {}
        for key, value in d_eq.items():
            if key not in _d_valid_parms:
                msg = (f"Wrong parameter: {key}; "
                       f"valid parameters are: {_d_valid_parms}")
                raise Exception(msg)
            d_eq_new[key] = cls.sh_value(key, value, _d_valid_parms)
        return d_eq_new
